/*
 * anim.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 * Anim is used for entities. Overlay is used for held items. If the
 * entity (creature) is holding an item, the entity graphic is drawn
 * first and the item graphic is drawn over the top.
 *
 */

#ifndef ANIM_HPP
#define ANIM_HPP

#include "map_support.hpp"
#include "platform/colour_change.hpp"

#include "boost/shared_ptr.hpp"
using namespace boost;

#include <iosfwd>
using namespace std;

class Graphic;
class ClientConfig;

class Anim {
public:
	explicit Anim(bool vbat=false) : id(0), vbat_mode(vbat)
		{ for (int i=0; i<4; ++i) for (int j=0; j<NFRAMES; ++j) g[i][j] = 0; }

	const Graphic * getGraphic(MapDirection facing, int frame, int x, int y,
							   int gvt) const;  // last 2 pars are used only for bats.
	const ColourChange & getColourChange(bool invuln) const
		{ return invuln ? cc_invulnerable : cc_normal; }
	
	void setGraphic(MapDirection d, int frm, const Graphic *g_) 
		{ g[d][frm] = g_; }
	void setColourChangeNormal(const ColourChange &cc)
		{ cc_normal = cc; }
	void setColourChangeInvulnerable(const ColourChange &cc)
		{ cc_invulnerable = cc; }

	//
	// serialization support
	//
	void serialize(ostream &, int new_id);
	Anim(istream &, const ClientConfig &);
	int getID() const { return id; }
	
private:
	enum { NFRAMES = 8 };
	const Graphic * g[4][NFRAMES]; // 4 facing directions
	ColourChange cc_normal, cc_invulnerable;
	int id;
	bool vbat_mode;
};


class Overlay {
public:
	Overlay() : id(0) { for (int i=0; i<4; ++i) g[i] = 0; }
	const Graphic * getGraphic(MapDirection d) const { return g[d]; }
	void setGraphic(MapDirection d, const Graphic *g_) { g[d] = g_; }

	// serialization
	void serialize(ostream &, int new_id);
	Overlay(istream &, const ClientConfig &);
	int getID() const { return id; }
	
private:
	const Graphic *g[4];
	int id;
};


// some standard frame numbers
enum AnimFrame { AF_NORMAL=0, AF_BACKSWING=1, AF_IMPACT=2, AF_PARRY=3, AF_THROW_BACK=4, AF_THROW_DOWN=5,
AF_XBOW=6, AF_XBOW_LOAD=7 };


// GetOverlayOffset
// input: entity's current facing and anim frame
// output: offset for the overlay graphic plus which overlay-direction to use.
// (if "show" is false then no overlay should be shown at all. used for throwing follow-through frame.)
void GetOverlayOffset(MapDirection facing, int frame, int &ofsx, int &ofsy, MapDirection &dir, bool &show);

// InitOverlayOffsets -- takes N_OVERLAY*3 ints. see cpp file for more info
enum { N_OVERLAY = 20 };
void InitOverlayOffsets(const int *);
void SerializeOverlayOffsets(ostream &);
void InitOverlayOffsets(istream &);

#endif
