/*
 * control.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef CONTROL_HPP
#define CONTROL_HPP

#include "map_support.hpp"

#include <vector>
using namespace std;

class Action;
class ClientConfig;
class Graphic;


//
// 'Controls' represent something a knight can do.
// (eg move, activate, pick up item).
//

class Control {
public:
	explicit Control(int new_id=0) : id(new_id), action(0), menu_graphic(0),
		menu_dir(D_NORTH), tap_pri(0), menu_special(0), cts(false) { }
	void construct(const Action *ac, const Graphic *g, MapDirection mdir, int tpri, bool c,
				   unsigned int ms)
		{ action = ac; menu_graphic = g; menu_dir = mdir; tap_pri = tpri; cts = c;
		  menu_special = ms; }

	// action to run when the control is selected.
	const Action * getAction() const { return action; }

	// if menu_graphic == 0 this control won't appear on the menu, else it will appear,
	// preferably in the slot given by getMenuDirection.
	const Graphic * getMenuGraphic() const { return menu_graphic; }
	MapDirection getMenuDirection() const { return menu_dir; }

	// if tap_priority > 0 then the action can be executed by tapping fire. (If more than one
	// possible action has this then the one with the highest tap_priority 'wins'.)
	int getTapPriority() const { return tap_pri; }

	// "continuous" controls (eg lockpicking) are executed for as long as you hold the button
	// down. "noncontinuous" controls (eg opening a door) execute once only.
	bool isContinuous() const { return cts; }

	// special properties
	// (there's only one at the moment!)
	enum MenuSpecial { MS_WEAK=1 };
	unsigned int getMenuSpecial() const { return menu_special; }

	// serialization
	void serialize(ostream &str, int new_id);
	Control(istream &str, const ClientConfig &, int new_id);
	int getID() const { return id; }
	
private:
	int id; 
	const Action *action;
	const Graphic *menu_graphic;
	MapDirection menu_dir;
	int tap_pri;
	unsigned int menu_special;
	bool cts;
};

//
// Standard controls
// (These are initialized in SetupStandardControls in control_actions.cpp)
//

enum StandardControlIndex {
	SC_ATTACK = 0,
	SC_MOVE = 4,
	SC_WITHDRAW = 8,
	SC_SUICIDE = 9, 
	NUM_STANDARD_CONTROLS = 10
};
extern Control * g_standard_controls[NUM_STANDARD_CONTROLS];


#endif
