/*
 * game_server.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 * This class is the entry point to the game engine itself. It acts as
 * a bridge between the config/setup code on one side, and the engine
 * on the other.
 *
 */

#ifndef GAME_SERVER_HPP
#define GAME_SERVER_HPP

#include "dungeon_map.hpp"

#include "boost/shared_ptr.hpp"
using namespace boost;

#include <vector>
using namespace std;

class DungeonGenerator;
class DungeonView;
class GameServerImpl;
class Menu;
class MenuSelections;
class Quest;
class ServerConfig;

class GameServer {
public:
	explicit GameServer(int nplyrs);
	~GameServer();

	// Initialize routine. (DungeonViews -- one per player -- must be
	// given.)
	void initialize(const ServerConfig &gc, const MenuSelections &ms,
					const vector<DungeonView*> dviews);

	// This is called to start things off.
	// (TODO) might like a more robust method, eg setting up a KtTask initially instead.
	// (then may not need this fn.)
	void spawnPlayers();

	// Update the game by moving GVT forward to a certain point. This
	// will call DungeonView functions to read controller state and
	// also to output graphics etc.
	void update(int new_gvt);
	
private:
	static void readMenu(const Menu &menu, const MenuSelections &ms, DungeonGenerator &dg,
						 vector<shared_ptr<Quest> > &quests);

private:
	shared_ptr<GameServerImpl> pimpl;
};

#endif
