/*
 * keyboard_controller.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 * KeyboardController: Controller that reads input from the Keyboard
 * Keys to use must be configured in the constructor.
 *
 */

#ifndef KEYBOARD_CONTROLLER_HPP
#define KEYBOARD_CONTROLLER_HPP

#include "controller.hpp"
#include "app/app2d.hpp"

#include "boost/scoped_ptr.hpp"
using namespace boost;

class KeyboardObserver;

class KeyboardController : public Controller {
public:
	KeyboardController(Keycode up_key_, Keycode right_key_, Keycode down_key_,
					   Keycode left_key_, Keycode fire_key_, Keycode suicide_key_,
					   App2D &app);
	virtual ~KeyboardController();

	virtual void get(MapDirection &dir, bool &centred, bool &fire_held,
					 bool &suicide_held) const;

	// currently we set F1--F12 to be the cheat keys.
	virtual int getCheatKey() const;
	
private:
	friend class KeyboardObserver;
	scoped_ptr<KeyboardObserver> obs;
	App2D &app;
	Keycode up_key, right_key, down_key, left_key, fire_key, suicide_key;
	bool up, right, down, left, fire, suicide;
	bool cheat[12];
};

#endif
