/*
 * gapi_graphic.hpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licenced for non-commercial use only. See LICENCE.txt for details.
 *
 * GAPI implementation of Graphic, CreateGraphic, DestroyGraphic
 *
 */

#ifndef GAPI_GRAPHIC_HPP
#define GAPI_GRAPHIC_HPP

#include "colour_change.hpp"
#include "graphic.hpp"

#include "boost/scoped_array.hpp"
using namespace boost;

class Graphic {
public:
	Graphic(const string &rname, int r, int g, int b, int hx, int hy);

	int getWidth() const { return width; }
	int getHeight() const { return height; }
	int getID() const { return id; }
	int getHX() const { return hx; }
	int getHY() const { return hy; }

	unsigned short int getPixel(int x, int y); // used by "sfont" code (NB coords rel to top left)
	
	// blit should not be used directly -- it is used internally from GapiDisplay routines.
	// for general use, use GapiDisplay::drawGraphic.
	void blit(int sx1, int sy1, int sx2, int sy2, int dx, int dy,
			  unsigned short *dst_ptr, int dpy_width, const ColourChange *cc) const;

private:
	int width, height, id;
	string rname;
	int r, g, b, hx, hy;
	scoped_array<unsigned short> pixel_data;
	bool use_col_key;
	unsigned short col_key;

	friend void SerializeGraphic(ostream &str, Graphic &g, int id);
};

#endif
